@@ -9,6 +9,8 @@ import numpy as np
 import pandas as pd
 from IPython.display import display
 
+from magnumapi.geometry.Geometry import Geometry
+from magnumapi.geometry.GeometryChange import GeometryChange
 from magnumapi.optimization.DesignVariable import GeneticDesignVariable
 from magnumapi.optimization.Logger import Logger
 from magnumapi.optimization.ModelExecutor import ModelExecutorFactory
@@ -29,13 +31,14 @@ class Optimization(ABC):
         self.n_pop = n_pop
         self.logger = Logger(output_subdirectory_dir, logger_rel_path)
         self.output_subdirectory_dir = output_subdirectory_dir
+        self.pop = []
 
     def optimize(self) -> None:
         """ Method executing the main optimization loop of the genetic algorithm
 
         """
         # initial population of random chromosomes
-        pop = self.initialize_population()
+        self.pop = self.initialize_population()
 
         # enumerate generations
         for gen in range(self.n_gen):
@@ -43,7 +46,7 @@ class Optimization(ABC):
             # decode population
             scores = []
 
-            for index, individual in enumerate(pop):
+            for index, individual in enumerate(self.pop):
                 print('\tIndividual:', index)
                 self.update_model(individual)
 
@@ -60,7 +63,7 @@ class Optimization(ABC):
             display(min_logger_df[min_logger_df.index == min_logger_df.index[-1]])
 
             # apply genetic operators of selection, crossover, and mutation
-            pop = self.update_generation(pop, scores)
+            self.pop = self.update_generation(self.pop, scores)
             self.logger.save_logger()
 
     @abstractmethod
@@ -280,7 +283,7 @@ class RoxieGeneticOptimization(GeneticOptimization):
     def __init__(self,
                  config: OptimizationConfig,
                  design_variables_df: pd.DataFrame,
-                 block_inputs: List[Dict],
+                 geometry: Geometry,
                  model_input_path: str = '',
                  is_script_executed: bool = True,
                  n_elite=2,
@@ -289,14 +292,14 @@ class RoxieGeneticOptimization(GeneticOptimization):
 
         :param config: a config for the genetic optimization
         :param design_variables_df: a dataframe with ranges of design variables
-        :param block_inputs: input block inputs to be updated; it is a list of dictionaries
+        :param geometry: input block inputs to be updated; it is a list of dictionaries
         :param model_input_path: a path to a model input json file
         :param is_script_executed: True if notebooks are executed as script, False otherwise
         :param n_elite: the number of individuals for elitism
         :param output_subdirectory_dir: a root subdirectory with output results
         """
         super().__init__(config, design_variables_df, n_elite, output_subdirectory_dir)
-        self.block_inputs = block_inputs
+        self.geometry = geometry
         self.model_input_path = model_input_path
         self.is_script_executed = is_script_executed
 
@@ -324,7 +327,7 @@ class RoxieGeneticOptimization(GeneticOptimization):
         # decoded_chromosome: a chromosome decoded into a dictionary with keys corresponding to parameter names
         # (variable and block index) mapping to parameter values
         decoded_chromosome = self.decode_individual(individual)
-        updated_block_inputs = self.update_model_parameters(decoded_chromosome)
+        updated_block_inputs = self.update_model_parameters_targeted(decoded_chromosome).to_dict()
 
         # write updated input to json
         with open(self.model_input_path, 'w') as file:
@@ -344,22 +347,49 @@ class RoxieGeneticOptimization(GeneticOptimization):
 
         return block_variable_value
 
-    def update_model_parameters(self, decoded_chromosome: Dict) -> List[Dict]:
+    def update_model_parameters(self, decoded_chromosome: Dict) -> Dict:
         """ Method updating model parameters given as ROXIE block definitions. Typically, it is a relative block
         definition.
 
         :param decoded_chromosome: a dictionary with keys containing a parameter name and block separated with a colon
         :return: an updated list of block definitions
         """
-        blocks_def_update = deepcopy(self.block_inputs)
+        geometry_model = deepcopy(self.geometry)
+        blocks_def_update = geometry_model.to_rel_geometry().to_dict()
         for block_variable, value in decoded_chromosome.items():
             if ':' in block_variable:
                 block, variable = block_variable.split(':')
                 block = int(block)
-                blocks_def_update[block - 1][variable] = value
+                blocks_def_update['block_defs'][block - 1][variable] = value
 
         # Remove empty blocks
-        return [block for block in blocks_def_update if block['nco'] > 0]
+        # ToDo: Update layer definition as well
+        block_defs = blocks_def_update['block_defs']
+        blocks_def_update['block_defs'] = [block for block in block_defs if block['nco'] > 0]
+        return blocks_def_update
+
+    def update_model_parameters_targeted(self, decoded_chromosome: Dict) -> Geometry:
+        geometry_rel = deepcopy(self.geometry)
+
+        # update phi_r
+        geometry_rel = GeometryChange.update_phi_r(geometry_rel, decoded_chromosome)
+
+        # update nco_r
+        geometry_rel = GeometryChange.update_nco_r(geometry_rel, decoded_chromosome)
+
+        # extract absolute geometry to correct radiality
+        geometry_abs = geometry_rel.to_abs_geometry()
+
+        # correct block radiality
+        geometry_abs = GeometryChange.calculate_radial_alpha(geometry_abs)
+
+        # extract relative geometry
+        geometry_rel = geometry_abs.to_rel_geometry()
+
+        # update alpha_rad_r
+        geometry_rel = GeometryChange.update_alpha_radial(geometry_rel, decoded_chromosome)
+
+        return geometry_rel
 
     def calculate_figures_of_merit(self, individual: List[int]) -> dict:
         """Method calculating figures of merit with scripts or notebooks (`papermill` and scrapbook packages).
