@@ -1,6 +1,6 @@
 from typing import List
 
-from magnumapi.geometry.CosThetaGeometry import HomogenizedCosThetaGeometry
+from magnumapi.geometry.Geometry import HomogenizedCosThetaGeometry
 
 
 class AnsysInputBuilder:
