@@ -3,19 +3,27 @@ from unittest.mock import patch
 
 import pandas as pd
 
-from magnumapi.geometry.CosThetaGeometry import HomogenizedCosThetaGeometry
+from magnumapi.geometry.Geometry import HomogenizedCosThetaGeometry
+from magnumapi.geometry.GeometryChange import GeometryChange
+from magnumapi.geometry.GeometryCheck import GeometryCheck
 from magnumapi.geometry.GeometryFactory import GeometryFactory
-from magnumapi.geometry.Line import Line
-from magnumapi.geometry.roxie.CableDatabase import CableDatabase
+from magnumapi.geometry.GeometryPlot import GeometryPlot
+from magnumapi.geometry.primitives.Line import Line
+from magnumapi.cadata.CableDatabase import CableDatabase
 from tests.resource_files import read_csv_as_pd, create_resources_file_path
 
 
 class CosThetaGeometryTest(unittest.TestCase):
+    layer_dct_defs = [
+        {"no": 1, "symm": 1, "typexy": 1, "blocks": [1, 2, 3, 4]},
+        {"no": 2, "symm": 1, "typexy": 1, "blocks": [5, 6, 7]},
+        {"no": 3, "symm": 1, "typexy": 1, "blocks": [8, 9, 10]},
+        {"no": 4, "symm": 1, "typexy": 1, "blocks": [11, 12]}]
 
     @patch("matplotlib.pyplot.show")
     def test_build_geometry_with_bare_absolute_angles_rectangle_roxie(self, mock_show=None):
         # arrange
-        blocks_def = [
+        blocks_dct_defs = [
             {'no': 1, 'radius': 25.0, 'alpha': 0, 'phi': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
              'condname': '16TOL77B', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0},
             {'no': 2, 'radius': 25.0, 'alpha': 26, 'phi': 23.0, 'nco': 5, 'type': 1, 'current': 13500,
@@ -45,7 +53,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_dct_defs, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -89,7 +97,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
 
         # assert
@@ -98,7 +106,7 @@ class CosThetaGeometryTest(unittest.TestCase):
 
         roxie_block_def = read_csv_as_pd(
             'resources/geometry/roxie/16T/reference/roxie_block_def_bare_absolute_angles_rectangle_ref.csv')
-        pd.testing.assert_frame_equal(roxie_block_def, geometry.to_roxie_df())
+        pd.testing.assert_frame_equal(roxie_block_def, geometry.to_block_df())
 
     @patch("matplotlib.pyplot.show")
     def test_build_geometry_with_bare_relative_angles_rectangle_roxie_to_df(self, mock_show=None):
@@ -133,7 +141,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -143,7 +151,7 @@ class CosThetaGeometryTest(unittest.TestCase):
 
         roxie_block_def = read_csv_as_pd(
             'resources/geometry/roxie/16T/reference/roxie_block_def_bare_relative_angles_rectangle_ref.csv')
-        pd.testing.assert_frame_equal(roxie_block_def, geometry.to_roxie_df())
+        pd.testing.assert_frame_equal(roxie_block_def, geometry.to_block_df())
 
         # assert
         if mock_show is not None:
@@ -181,7 +189,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -192,17 +200,19 @@ class CosThetaGeometryTest(unittest.TestCase):
     @patch("matplotlib.pyplot.show")
     def test_build_geometry_with_roxie_bare_absolute_angles_rectangle_csv(self, mock_show=None):
         # arrange
-        df_path = create_resources_file_path('resources/geometry/roxie/16T/bare_absolute_block_data_rectangle.csv')
-        cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
+        root_dir = 'resources/geometry/roxie/16T/'
+        block_df_path = create_resources_file_path('%sbare_absolute_block_data_rectangle.csv' % root_dir)
+        layer_df_path = create_resources_file_path('%slayer_data.csv' % root_dir)
+        cadata_path = create_resources_file_path('%sroxieold_2.cadata' % root_dir)
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_csv(df_path, cadata)
+        geometry = GeometryFactory.init_with_csv(block_df_path, layer_df_path, cadata)
         geometry.build_blocks()
         geometry.plot_blocks(xlim=(0, 80), ylim=(0, 80))
 
         # assert
-        geometry_ref = read_csv_as_pd('resources/geometry/roxie/16T/reference/bare_absolute_angles_rectangle_ref.csv')
+        geometry_ref = read_csv_as_pd('%sreference/bare_absolute_angles_rectangle_ref.csv' % root_dir)
         pd.testing.assert_frame_equal(geometry_ref, geometry.to_df())
 
         # assert
@@ -213,11 +223,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_bare_absolute_angles_rectangle_df(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/bare_absolute_block_data_rectangle.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks(xlim=(0, 80), ylim=(0, 80))
 
@@ -232,17 +243,19 @@ class CosThetaGeometryTest(unittest.TestCase):
     @patch("matplotlib.pyplot.show")
     def test_build_geometry_with_roxie_bare_relative_angles_rectangle_csv(self, mock_show=None):
         # arrange
-        df_path = create_resources_file_path('resources/geometry/roxie/16T/bare_relative_block_data_rectangle.csv')
-        cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
+        root_dir = 'resources/geometry/roxie/16T/'
+        block_df_path = create_resources_file_path('%sbare_relative_block_data_rectangle.csv' % root_dir)
+        layer_df_path = create_resources_file_path('%slayer_data.csv' % root_dir)
+        cadata_path = create_resources_file_path('%sroxieold_2.cadata' % root_dir)
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_csv(df_path, cadata)
+        geometry = GeometryFactory.init_with_csv(block_df_path, layer_df_path, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
         # assert
-        geometry_ref = read_csv_as_pd('resources/geometry/roxie/16T/reference/bare_relative_angles_rectangle_ref.csv')
+        geometry_ref = read_csv_as_pd('%sreference/bare_relative_angles_rectangle_ref.csv' % root_dir)
         pd.testing.assert_frame_equal(geometry_ref, geometry.to_df())
 
         # assert
@@ -253,11 +266,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_bare_relative_angles_rectangle_df(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/bare_relative_block_data_rectangle.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -289,11 +303,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_bare_absolute_angles_trapezoid(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/bare_absolute_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -309,11 +324,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_bare_relative_angles_trapezoid(self, mock_show=None):
         # arrangee
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/bare_relative_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -329,11 +345,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_ins_relative_angles_rectangle(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/ins_relative_block_data_rectangle.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -349,11 +366,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_ins_absolute_angles_trapezoid(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/ins_absolute_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -369,11 +387,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_build_geometry_with_roxie_ins_relative_angles_trapezoid(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/ins_relative_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
         geometry.plot_bare_blocks()
@@ -418,55 +437,12 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
         # assert
-        self.assertFalse(geometry.is_outside_of_first_quadrant())
-
-        # assert
-        if mock_show is not None:
-            mock_show.assert_called()
-
-    @patch("matplotlib.pyplot.show")
-    def test_check_geometry_consistency_first_quadrant_pass(self, mock_show=None):
-        # arrange
-        blocks_def = [{'no': 1, 'radius': 25.0, 'alpha': 0, 'phi': 0.57294, 'nco': 4, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 2, 'radius': 25.0, 'alpha': 26, 'phi': 23.0, 'nco': 5, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 3, 'radius': 25.0, 'alpha': 47, 'phi': 50.8, 'nco': 2, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 4, 'radius': 25.0, 'alpha': 66, 'phi': 65.5, 'nco': 10, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 5, 'radius': 39.0, 'alpha': 0, 'phi': 0.36728, 'nco': 7, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 6, 'radius': 39.0, 'alpha': 35, 'phi': 26.0, 'nco': 10, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 7, 'radius': 39.0, 'alpha': 54, 'phi': 61.0, 'nco': 2, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 8, 'radius': 53.0, 'alpha': 0, 'phi': 0.27026, 'nco': 18, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 9, 'radius': 53.0, 'alpha': 30, 'phi': 31.3, 'nco': 9, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 10, 'radius': 53.0, 'alpha': 50, 'phi': 57.0, 'nco': 2, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 11, 'radius': 67.45, 'alpha': 0, 'phi': 0.21236, 'nco': 28, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1},
-                      {'no': 12, 'radius': 67.45, 'alpha': 30, 'phi': 37.9, 'nco': 11, 'condname': '16TOL77B',
-                       'current': 13500, 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0, 'type': 1}]
-
-        cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
-        cadata = CableDatabase.read_cadata(cadata_path)
-
-        # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
-        geometry.build_blocks()
-        geometry.plot_blocks()
-
-        # assert
-        self.assertFalse(geometry.is_outside_of_first_quadrant())
+        self.assertFalse(GeometryCheck.is_outside_of_first_quadrant(geometry))
 
         # assert
         if mock_show is not None:
@@ -485,7 +461,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         geometry.plot_blocks()
 
         # assert
-        self.assertFalse(geometry.is_outside_of_first_quadrant())
+        self.assertFalse(GeometryCheck.is_outside_of_first_quadrant(geometry))
 
         # assert
         if mock_show is not None:
@@ -495,17 +471,18 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_check_geometry_consistency_too_sharp_wedge_pass(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/ins_relative_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
         geometry.plot_bare_blocks()
 
         # assert
-        is_wedge_tip_too_sharp = geometry.is_wedge_tip_too_sharp(min_value_in_mm=2)
+        is_wedge_tip_too_sharp = GeometryCheck.is_wedge_tip_too_sharp(geometry, min_value_in_mm=2)
 
         # check if a wedge is not too sharp
         self.assertEqual(False, is_wedge_tip_too_sharp)
@@ -518,17 +495,18 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_check_geometry_consistency_too_sharp_wedge_fail(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/ins_relative_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
         geometry.plot_bare_blocks()
 
         # assert
-        is_wedge_tip_too_sharp = geometry.is_wedge_tip_too_sharp(min_value_in_mm=5)
+        is_wedge_tip_too_sharp = GeometryCheck.is_wedge_tip_too_sharp(geometry, min_value_in_mm=5)
 
         # check if a wedge is not too sharp
         self.assertTrue(is_wedge_tip_too_sharp)
@@ -568,12 +546,12 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
         # assert
-        self.assertFalse(geometry.are_turns_overlapping())
+        self.assertFalse(GeometryCheck.are_turns_overlapping(geometry))
 
         # assert
         if mock_show is not None:
@@ -611,12 +589,12 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
         # assert
-        self.assertTrue(geometry.are_turns_overlapping())
+        self.assertTrue(GeometryCheck.are_turns_overlapping(geometry))
 
         # assert
         if mock_show is not None:
@@ -626,11 +604,12 @@ class CosThetaGeometryTest(unittest.TestCase):
     def test_check_geometry_ansys(self, mock_show=None):
         # arrange
         block_df = read_csv_as_pd('resources/geometry/roxie/16T/ins_relative_block_data_trapezoid.csv')
+        layer_df = read_csv_as_pd('resources/geometry/roxie/16T/layer_data.csv')
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_df(block_df, cadata)
+        geometry = GeometryFactory.init_with_df(block_df, layer_df, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
         geometry.plot_bare_blocks()
@@ -663,12 +642,13 @@ class CosThetaGeometryTest(unittest.TestCase):
             {'no': 1, 'radius': 25, 'alpha': 10, 'phi': 5, 'nco': 3, 'type': 1, 'current': 13500,
              'condname': '16TOL78B',
              'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        layer_def = [{"no": 1, "symm": 1, "typexy": 1, "blocks": [1]}]
 
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, layer_def, cadata)
 
         geometry.build_blocks()
         geometry.plot_blocks()
@@ -800,12 +780,13 @@ class CosThetaGeometryTest(unittest.TestCase):
         blocks_def = [
             {'no': 1, 'radius': 25, 'alpha_r': 10, 'phi_r': 5, 'nco': 3, 'type': 1, 'current': 13500,
              'condname': '16TOL78B', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        layer_def = [{"no": 1, "symm": 1, "typexy": 1, "blocks": [1]}]
 
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, layer_def, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
         # Conductor number: 1
@@ -960,7 +941,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -1009,7 +990,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(blocks_def, self.layer_dct_defs, cadata)
         geometry.build_blocks()
         geometry.plot_blocks()
 
@@ -1025,14 +1006,15 @@ class CosThetaGeometryTest(unittest.TestCase):
     @patch("plotly.graph_objects.Figure.show")
     def test_simplified_magnet_input_plotly_blocks(self, mock_show=None):
         # arrange
-        blocks_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
-                       'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        block_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
+                      'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        layer_def = [{"no": 1, "symm": 1, "typexy": 1, "blocks": [1]}]
 
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(block_def, layer_def, cadata)
         geometry.build_blocks()
         geometry.plotly_blocks()
 
@@ -1043,14 +1025,15 @@ class CosThetaGeometryTest(unittest.TestCase):
     @patch("plotly.graph_objects.Figure.show")
     def test_simplified_magnet_input_plotly_homogenized_blocks(self, mock_show=None):
         # arrange
-        blocks_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
-                       'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        block_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
+                      'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        layer_def = [{"no": 1, "symm": 1, "typexy": 1, "blocks": [1]}]
 
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(block_def, layer_def, cadata)
         geometry.build_blocks()
 
         homo_geometry = HomogenizedCosThetaGeometry.with_cos_theta_geometry(geometry)
@@ -1063,16 +1046,17 @@ class CosThetaGeometryTest(unittest.TestCase):
 
     def test_simplified_magnet_input_display_table(self):
         # arrange
-        blocks_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
+        block_def = [{'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 4, 'type': 1, 'current': 13500,
                        'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+        layer_def = [{"no": 1, "symm": 1, "typexy": 1, "blocks": [1]}]
 
         cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
         cadata = CableDatabase.read_cadata(cadata_path)
 
         # act
-        geometry = GeometryFactory.init_with_dict(blocks_def, cadata)
+        geometry = GeometryFactory.init_with_dict(block_def, layer_def, cadata)
         geometry.build_blocks()
-        geometry.display_definition_table(geometry.to_df())
+        GeometryPlot.display_definition_table(geometry.to_df())
 
     def test_with_roxie_absolute_angle_definition_json(self):
         # arrange
@@ -1135,7 +1119,7 @@ class CosThetaGeometryTest(unittest.TestCase):
         geometry.build_blocks()
         geometry.plotly_blocks(figsize=(90, 90), xlim=(0, 85))
 
-        geometry.correct_radiality()
+        geometry = GeometryChange.calculate_radial_alpha(geometry)
 
         # Build corrected blocks
         geometry.build_blocks()
