@@ -3,8 +3,9 @@ from unittest import TestCase
 import pandas as pd
 import numpy as np
 
+from magnumapi.geometry.GeometryChange import GeometryChange
 from magnumapi.geometry.GeometryFactory import GeometryFactory
-from magnumapi.geometry.roxie.CableDatabase import CableDatabase
+from magnumapi.cadata.CableDatabase import CableDatabase
 from magnumapi.optimization.GeneticOptimization import GeneticOptimization, RoxieGeneticOptimization
 from tests.resource_files import create_resources_file_path
 
@@ -15,7 +16,7 @@ optimization_cfg = GeneticOptimization.initialize_config(json_file_path)
 json_path = create_resources_file_path('resources/geometry/roxie/16T/16T_rel.json')
 cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
 cadata = CableDatabase.read_cadata(cadata_path)
-block_inputs = GeometryFactory.read_json_file(json_path)
+geometry = GeometryFactory.init_with_json(json_path, cadata)
 
 
 class TestRoxieGeneticOptimization(TestCase):
@@ -23,7 +24,7 @@ class TestRoxieGeneticOptimization(TestCase):
     def setUp(self) -> None:
         self.gen_opt = RoxieGeneticOptimization(config=optimization_cfg,
                                                 design_variables_df=pd.read_csv(csv_file_path),
-                                                block_inputs=block_inputs,
+                                                geometry=geometry,
                                                 model_input_path='',
                                                 is_script_executed=True,
                                                 output_subdirectory_dir='')
@@ -115,10 +116,10 @@ class TestRoxieGeneticOptimization(TestCase):
         # act
         pop = self.gen_opt.initialize_population()
         chromosome = self.gen_opt.decode_individual(pop[0])
-        blocks_update_def = self.gen_opt.update_model_parameters(chromosome)
+        block_layer_defs = self.gen_opt.update_model_parameters(chromosome)
 
         # assert
-        blocks_def_ref = [
+        block_defs_ref = {'block_defs': [
             {'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 3, 'type': 1, 'current': 13500,
              'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0},
             {'no': 2, 'radius': 25.0, 'alpha_r': 3.59375, 'phi_r': 5.953125, 'nco': 2, 'type': 1, 'current': 13500,
@@ -138,9 +139,13 @@ class TestRoxieGeneticOptimization(TestCase):
             {'no': 11, 'radius': 67.45, 'alpha_r': 0, 'phi_r': 0.21236, 'nco': 30, 'type': 1, 'current': 13500,
              'condname': '16TOL8', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0},
             {'no': 12, 'radius': 67.45, 'alpha_r': 7.8125, 'phi_r': 6.390625, 'nco': 12, 'type': 1, 'current': 13500,
-             'condname': '16TOL8', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
+             'condname': '16TOL8', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}],
+            'layer_defs': [{'no': 1, 'symm': 1, 'typexy': 1, 'blocks': [1, 2, 3, 4]},
+                           {'no': 2, 'symm': 1, 'typexy': 1, 'blocks': [5, 6, 7]},
+                           {'no': 3, 'symm': 1, 'typexy': 1, 'blocks': [8, 9, 10]},
+                           {'no': 3, 'symm': 1, 'typexy': 1, 'blocks': [11, 12]}]}
 
-        self.assertListEqual(blocks_def_ref, blocks_update_def)
+        assert block_defs_ref['block_defs'] == block_layer_defs['block_defs']
 
     def test_update_parameters_multi_index(self):
         # arrange
@@ -153,10 +158,10 @@ class TestRoxieGeneticOptimization(TestCase):
         # act
         pop = self.gen_opt.initialize_population()
         chromosome = self.gen_opt.decode_individual(pop[0])
-        blocks_update_def = self.gen_opt.update_model_parameters(chromosome)
+        block_layer_defs = self.gen_opt.update_model_parameters(chromosome)
 
         # assert
-        blocks_def_ref = [
+        block_defs_ref = [
             {'no': 1, 'radius': 25.0, 'alpha_r': 0, 'phi_r': 0.57294, 'nco': 3, 'type': 1, 'current': 12257.8125,
              'condname': '16TIL9', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0},
             {'no': 2, 'radius': 25.0, 'alpha_r': 3.59375, 'phi_r': 5.953125, 'nco': 2, 'type': 1, 'current': 12257.8125,
@@ -178,4 +183,107 @@ class TestRoxieGeneticOptimization(TestCase):
             {'no': 12, 'radius': 67.45, 'alpha_r': 7.8125, 'phi_r': 6.390625, 'nco': 12, 'type': 1,
              'current': 12257.8125, 'condname': '16TOL8', 'n1': 2, 'n2': 20, 'imag': 0, 'turn': 0}]
 
-        self.assertListEqual(blocks_def_ref, blocks_update_def)
+        self.assertListEqual(block_defs_ref, block_layer_defs['block_defs'])
+
+    def test_correct_missing_blocks_in_block_and_layer_definitions(self):
+        # arrange
+        chromosome = {'1:nco_r': 2,
+                      '2:nco_r': -3}
+        json_path = create_resources_file_path('resources/geometry/roxie/16T/16T_rel.json')
+        cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
+        cadata = CableDatabase.read_cadata(cadata_path)
+        geometry = GeometryFactory.init_with_json(json_path, cadata)
+
+        # act
+        # # update number of turns per block_index
+        geometry = GeometryChange.update_nco_r(geometry, chromosome)
+
+        assert len(geometry.blocks) == 11
+        assert geometry.layer_defs[0].blocks == [1, 3, 4]
+        assert geometry.layer_defs[1].blocks == [5, 6, 7]
+        assert geometry.layer_defs[2].blocks == [8, 9, 10]
+        assert geometry.layer_defs[3].blocks == [11, 12]
+
+    def test_update_targeted_optimization_input_geometry(self):
+        np.random.seed(1)
+
+        # act
+        path_str = 'resources/optimization/optim_input_enlarged_targeted_optimization.csv'
+        csv_global_file_path = create_resources_file_path(path_str)
+        df = pd.read_csv(csv_global_file_path)
+        self.gen_opt.design_variables = RoxieGeneticOptimization.initialize_design_variables(df)
+        pop = self.gen_opt.initialize_population()
+        chromosome = self.gen_opt.decode_individual(pop[0])
+
+        # read an input file with geometry definition
+        cadata_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
+        cadata = CableDatabase.read_cadata(cadata_path)
+        json_ref_path = create_resources_file_path('resources/geometry/roxie/16T/16T_rel.json')
+
+        # initialize relative geometry
+        geometry_rel = GeometryFactory.init_with_json(json_ref_path, cadata)
+
+        # update phi_r
+        geometry_rel = GeometryChange.update_phi_r(geometry_rel, chromosome)
+
+        # update nco_r
+        geometry_rel = GeometryChange.update_nco_r(geometry_rel, chromosome)
+
+        # extract absolute geometry
+        geometry_abs = geometry_rel.to_abs_geometry()
+
+        # correct block radiality
+        geometry_abs = GeometryChange.calculate_radial_alpha(geometry_abs)
+
+        # extract relative geometry
+        geometry_rel = geometry_abs.to_rel_geometry()
+
+        # update alpha_rad_r with
+        geometry_rel = GeometryChange.update_alpha_radial(geometry_rel, chromosome)
+
+        # assert that the number of turns per layer is the same
+        assert 13 == sum([block.block_def.nco for block in geometry_rel.blocks[:4]])
+        assert 19 == sum([block.block_def.nco for block in geometry_rel.blocks[4:7]])
+        assert 29 == sum([block.block_def.nco for block in geometry_rel.blocks[7:10]])
+        assert 39 == sum([block.block_def.nco for block in geometry_rel.blocks[10:]])
+
+    def test_update_model_parameters_targeted(self):
+        np.random.seed(1)
+
+        # act
+        path_str = 'resources/optimization/optim_input_enlarged_targeted_optimization.csv'
+        csv_global_file_path = create_resources_file_path(path_str)
+        df = pd.read_csv(csv_global_file_path)
+        self.gen_opt.design_variables = RoxieGeneticOptimization.initialize_design_variables(df)
+        pop = self.gen_opt.initialize_population()
+        chromosome = self.gen_opt.decode_individual(pop[0])
+        geometry_rel = self.gen_opt.update_model_parameters_targeted(chromosome)
+
+        # assert that the number of turns per layer is the same
+        assert 13 == sum([block.block_def.nco for block in geometry_rel.blocks[:4]])
+        assert 19 == sum([block.block_def.nco for block in geometry_rel.blocks[4:7]])
+        assert 29 == sum([block.block_def.nco for block in geometry_rel.blocks[7:10]])
+        assert 39 == sum([block.block_def.nco for block in geometry_rel.blocks[10:]])
+
+    def test_update_model_parameters_targeted_error(self):
+        np.random.seed(1)
+
+        # act
+        path_str = 'resources/optimization/optim_input_enlarged_targeted_optimization.csv'
+        csv_global_file_path = create_resources_file_path(path_str)
+        df = pd.read_csv(csv_global_file_path)
+        self.gen_opt.design_variables = RoxieGeneticOptimization.initialize_design_variables(df)
+        pop = [[1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 1,
+                1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1,
+                0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1,
+                0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0]]
+
+        chromosome = self.gen_opt.decode_individual(pop[0])
+        geometry_rel = self.gen_opt.update_model_parameters_targeted(chromosome)
+
+        # assert that the number of turns per layer is the same
+        assert 13 == sum([block.block_def.nco for block in geometry_rel.blocks[:3]])
+        assert 19 == sum([block.block_def.nco for block in geometry_rel.blocks[3:6]])
+        assert 29 == sum([block.block_def.nco for block in geometry_rel.blocks[6:9]])
+        assert 39 == sum([block.block_def.nco for block in geometry_rel.blocks[9:]])
+
